@@ -13,18 +13,25 @@ except ImportError:
     # python2
     pass
 
+try:
+    # Enums are supported for Python 3.4+ and Python 2.7 with enum34 package installed
+    from enum import Enum, EnumMeta
+except ImportError:
+    # If enum is not available, create dummy classes for type checks
+    class Enum:
+        pass
+
+    class EnumMeta:
+        pass
+
 import inspect
 import itertools
 import logging
-import warnings
 
 from collections import OrderedDict, defaultdict, deque
 from functools import partial
 from six import string_types
 
-# make deprecation warnings of transition visible for module users
-warnings.filterwarnings(action='default', message=r"Starting from transitions version 0\.6\.0 .*")
-
 _LOGGER = logging.getLogger(__name__)
 _LOGGER.addHandler(logging.NullHandler())
 
@@ -39,24 +46,26 @@ def listify(obj):
     """
     if obj is None:
         return []
-    return obj if isinstance(obj, (list, tuple)) else [obj]
+
+    return obj if isinstance(obj, (list, tuple, EnumMeta)) else [obj]
 
 
-def _get_trigger(model, trigger_name, *args, **kwargs):
+def _get_trigger(model, machine, trigger_name, *args, **kwargs):
     """Convenience function added to the model to trigger events by name.
-
     Args:
         model (object): Model with assigned event trigger.
+        machine (Machine): The machine containing the evaluated events.
         trigger_name (str): Name of the trigger to be called.
         *args: Variable length argument list which is passed to the triggered event.
         **kwargs: Arbitrary keyword arguments which is passed to the triggered event.
     Returns:
         bool: True if a transitions has been conducted or the trigger event has been queued.
     """
-    func = getattr(model, trigger_name, None)
-    if func:
-        return func(*args, **kwargs)
-    raise AttributeError("Model has no trigger named '%s'" % trigger_name)
+    try:
+        event = machine.events[trigger_name]
+    except KeyError:
+        raise AttributeError("Do not know event named '%s'." % trigger_name)
+    return event.trigger(model, *args, **kwargs)
 
 
 def _prep_ordered_arg(desired_length, arguments=None):
@@ -85,7 +94,7 @@ class State(object):
     Attributes:
         name (str): State name which is also assigned to the model(s).
         on_enter (list): Callbacks executed when a state is entered.
-        on_exit (list): Callbacks executed when a state is entered.
+        on_exit (list): Callbacks executed when a state is exited.
         ignore_invalid_triggers (bool): Indicates if unhandled/invalid triggers should raise an exception.
     """
 
@@ -94,25 +103,36 @@ class State(object):
     dynamic_methods = ['on_enter', 'on_exit']
 
     def __init__(self, name, on_enter=None, on_exit=None,
-                 ignore_invalid_triggers=False):
+                 ignore_invalid_triggers=None):
         """
         Args:
-            name (string): The name of the state
-            on_enter (string, list): Optional callable(s) to trigger when a
+            name (str): The name of the state
+            on_enter (str or list): Optional callable(s) to trigger when a
                 state is entered. Can be either a string providing the name of
                 a callable, or a list of strings.
-            on_exit (string, list): Optional callable(s) to trigger when a
+            on_exit (str or list): Optional callable(s) to trigger when a
                 state is exited. Can be either a string providing the name of a
                 callable, or a list of strings.
             ignore_invalid_triggers (Boolean): Optional flag to indicate if
                 unhandled/invalid triggers should raise an exception
 
         """
-        self.name = name
+        self._name = name
         self.ignore_invalid_triggers = ignore_invalid_triggers
         self.on_enter = listify(on_enter) if on_enter else []
         self.on_exit = listify(on_exit) if on_exit else []
 
+    @property
+    def name(self):
+        if isinstance(self._name, Enum):
+            return self._name.name
+        else:
+            return self._name
+
+    @property
+    def value(self):
+        return self._name
+
     def enter(self, event_data):
         """ Triggered when a state is entered. """
         _LOGGER.debug("%sEntering state %s. Processing callbacks...", event_data.machine.name, self.name)
@@ -130,9 +150,9 @@ class State(object):
     def add_callback(self, trigger, func):
         """ Add a new enter or exit callback.
         Args:
-            trigger (string): The type of triggering event. Must be one of
+            trigger (str): The type of triggering event. Must be one of
                 'enter' or 'exit'.
-            func (string): The name of the callback function.
+            func (str): The name of the callback function.
         """
         callback_list = getattr(self, 'on_' + trigger)
         callback_list.append(func)
@@ -154,7 +174,7 @@ class Condition(object):
     def __init__(self, func, target=True):
         """
         Args:
-            func (string): Name of the condition-checking callable
+            func (str): Name of the condition-checking callable
             target (bool): Indicates the target state--i.e., when True,
                 the condition-checking callback should return True to pass,
                 and when False, the callback should return False to pass.
@@ -270,15 +290,15 @@ class Transition(object):
     def _change_state(self, event_data):
         event_data.machine.get_state(self.source).exit(event_data)
         event_data.machine.set_state(self.dest, event_data.model)
-        event_data.update(event_data.model)
+        event_data.update(getattr(event_data.model, event_data.machine.model_attribute))
         event_data.machine.get_state(self.dest).enter(event_data)
 
     def add_callback(self, trigger, func):
         """ Add a new before, after, or prepare callback.
         Args:
-            trigger (string): The type of triggering event. Must be one of
+            trigger (str): The type of triggering event. Must be one of
                 'before', 'after' or 'prepare'.
-            func (string): The name of the callback function.
+            func (str): The name of the callback function.
         """
         callback_list = getattr(self, trigger)
         callback_list.append(func)
@@ -327,13 +347,15 @@ class EventData(object):
         self.error = None
         self.result = False
 
-    def update(self, model):
-        """ Updates the current State of a model to accurately reflect the Machine.
+    def update(self, state):
+        """ Updates the EventData object with the passed state.
 
         Attributes:
-            model (object): The updated model which gets the updated state assigned to its attribute `state`.
+            state (State, str or Enum): The state object, enum member or string to assign to EventData.
         """
-        self.state = self.machine.get_state(model.state)
+
+        if not isinstance(state, State):
+            self.state = self.machine.get_state(state)
 
     def __repr__(self):
         return "<%s('%s', %s)@%s>" % (type(self).__name__, self.state,
@@ -348,7 +370,7 @@ class Event(object):
     def __init__(self, name, machine):
         """
         Args:
-            name (string): The name of the event, which is also the name of the
+            name (str): The name of the event, which is also the name of the
                 triggering callable (e.g., 'advance' implies an advance()
                 method).
             machine (Machine): The current Machine instance.
@@ -386,11 +408,13 @@ class Event(object):
         """ Internal trigger function called by the ``Machine`` instance. This should not
         be called directly but via the public method ``Machine.trigger``.
         """
-        state = self.machine.get_state(model.state)
+        state = self.machine.get_model_state(model)
         if state.name not in self.transitions:
             msg = "%sCan't trigger event %s from state %s!" % (self.machine.name, self.name,
                                                                state.name)
-            if state.ignore_invalid_triggers:
+            ignore = state.ignore_invalid_triggers if state.ignore_invalid_triggers is not None \
+                else self.machine.ignore_invalid_triggers
+            if ignore:
                 _LOGGER.warning(msg)
                 return False
             else:
@@ -424,9 +448,9 @@ class Event(object):
     def add_callback(self, trigger, func):
         """ Add a new before or after callback to all available transitions.
         Args:
-            trigger (string): The type of triggering event. Must be one of
+            trigger (str): The type of triggering event. Must be one of
                 'before', 'after' or 'prepare'.
-            func (string): The name of the callback function.
+            func (str): The name of the callback function.
         """
         for trans in itertools.chain(*self.transitions.values()):
             trans.add_callback(trigger, func)
@@ -472,17 +496,17 @@ class Machine(object):
                  send_event=False, auto_transitions=True,
                  ordered_transitions=False, ignore_invalid_triggers=None,
                  before_state_change=None, after_state_change=None, name=None,
-                 queued=False, prepare_event=None, finalize_event=None, **kwargs):
+                 queued=False, prepare_event=None, finalize_event=None, model_attribute='state', **kwargs):
         """
         Args:
             model (object or list): The object(s) whose states we want to manage. If 'self',
                 the current Machine instance will be used the model (i.e., all
                 triggering events will be attached to the Machine itself). Note that an empty list
                 is treated like no model.
-            states (list): A list of valid states. Each element can be either a
-                string or a State instance. If string, a new generic State
-                instance will be created that has the same name as the string.
-            initial (string or State): The initial state of the passed model[s].
+            states (list or Enum): A list or enumeration of valid states. Each list element can be either a
+                string, an enum member or a State instance. If string or enum member, a new generic State
+                instance will be created that is named according to the string or enum member's name.
+            initial (str, Enum or State): The initial state of the passed model[s].
             transitions (list): An optional list of transitions. Each element
                 is a dictionary of named arguments to be passed onto the
                 Transition initializer.
@@ -520,11 +544,6 @@ class Machine(object):
             **kwargs additional arguments passed to next class in MRO. This can be ignored in most cases.
         """
 
-        if kwargs.pop('add_self', None) is not None:
-            warnings.warn("Starting from transitions version 0.6.0 'add_self' is no longer"
-                          "supported. To add the machine as a model use the new default "
-                          "value model='self' instead.", DeprecationWarning)
-
         # calling super in case `Machine` is used as a mix in
         # all keyword arguments should be consumed by now if this is not the case
         try:
@@ -551,6 +570,7 @@ class Machine(object):
         self.after_state_change = after_state_change
         self.finalize_event = finalize_event
         self.name = name + ": " if name is not None else ""
+        self.model_attribute = model_attribute
 
         self.models = []
 
@@ -582,7 +602,7 @@ class Machine(object):
         for mod in models:
             mod = self if mod == 'self' else mod
             if mod not in self.models:
-                self._checked_assignment(mod, 'trigger', partial(_get_trigger, mod))
+                self._checked_assignment(mod, 'trigger', partial(_get_trigger, mod, self))
 
                 for trigger, _ in self.events.items():
                     self._add_trigger_to_model(trigger, mod)
@@ -627,9 +647,10 @@ class Machine(object):
                 assert self._has_state(value)
             self._initial = value.name
         else:
-            if value not in self.states:
-                self.add_state(value)
-            self._initial = value
+            state_name = value.name if isinstance(value, Enum) else value
+            if state_name not in self.states:
+                self.add_state(state_name)
+            self._initial = state_name
 
     @property
     def has_queue(self):
@@ -691,6 +712,8 @@ class Machine(object):
 
     def get_state(self, state):
         """ Return the State instance with the passed name. """
+        if isinstance(state, Enum):
+            state = state.name
         if state not in self.states:
             raise ValueError("State '%s' is not a registered state." % state)
         return self.states[state]
@@ -706,15 +729,22 @@ class Machine(object):
         Returns:
             bool: Whether the model's current state is state.
         """
-        return model.state == state
+        return getattr(model, self.model_attribute) == state
+
+    def get_model_state(self, model):
+        return self.get_state(getattr(model, self.model_attribute))
 
     def set_state(self, state, model=None):
-        """ Set the current state. """
-        if isinstance(state, string_types):
-            state = self.get_state(state)
+        """
+            Set the current state.
+        Args:
+            state (str or Enum): value of setted state
+        """
+        state = self.get_state(state)
         models = self.models if model is None else listify(model)
+
         for mod in models:
-            mod.state = state.name
+            setattr(mod, self.model_attribute, state.value)
 
     def add_state(self, *args, **kwargs):
         """ Alias for add_states. """
@@ -724,13 +754,12 @@ class Machine(object):
                    ignore_invalid_triggers=None, **kwargs):
         """ Add new state(s).
         Args:
-            states (list, string, dict, or State): a list, a State instance, the
-                name of a new state, or a dict with keywords to pass on to the
-                State initializer. If a list, each element can be of any of the
-                latter three types.
-            on_enter (string or list): callbacks to trigger when the state is
+            states (list, str, dict, Enum or State): a list, a State instance, the
+                name of a new state, an enumeration (member) or a dict with keywords to pass on to the
+                State initializer. If a list, each element can be a string, State or enumeration member.
+            on_enter (str or list): callbacks to trigger when the state is
                 entered. Only valid if first argument is string.
-            on_exit (string or list): callbacks to trigger when the state is
+            on_exit (str or list): callbacks to trigger when the state is
                 exited. Only valid if first argument is string.
             ignore_invalid_triggers: when True, any calls to trigger methods
                 that are not valid for the present state (e.g., calling an
@@ -749,8 +778,9 @@ class Machine(object):
             ignore = self.ignore_invalid_triggers
 
         states = listify(states)
+
         for state in states:
-            if isinstance(state, string_types):
+            if isinstance(state, (string_types, Enum)):
                 state = self._create_state(
                     state, on_enter=on_enter, on_exit=on_exit,
                     ignore_invalid_triggers=ignore, **kwargs)
@@ -767,7 +797,7 @@ class Machine(object):
                 self.add_transition('to_%s' % state, self.wildcard_all, state)
 
     def _add_model_to_state(self, state, model):
-        self._checked_assignment(model, 'is_%s' % state.name, partial(self.is_state, state.name, model))
+        self._checked_assignment(model, 'is_%s' % state.name, partial(self.is_state, state.value, model))
 
         # Add dynamic method callbacks (enter/exit) if there are existing bound methods in the model
         # except if they are already mentioned in 'on_enter/exit' of the defined state
@@ -801,45 +831,47 @@ class Machine(object):
                        unless=None, before=None, after=None, prepare=None, **kwargs):
         """ Create a new Transition instance and add it to the internal list.
         Args:
-            trigger (string): The name of the method that will trigger the
+            trigger (str): The name of the method that will trigger the
                 transition. This will be attached to the currently specified
                 model (e.g., passing trigger='advance' will create a new
                 advance() method in the model that triggers the transition.)
-            source(string or list): The name of the source state--i.e., the state we
+            source(str or list): The name of the source state--i.e., the state we
                 are transitioning away from. This can be a single state, a
                 list of states or an asterisk for all states.
-            dest (string): The name of the destination State--i.e., the state
+            dest (str): The name of the destination State--i.e., the state
                 we are transitioning into. This can be a single state or an
                 equal sign to specify that the transition should be reflexive
                 so that the destination will be the same as the source for
                 every given source. If dest is None, this transition will be
                 an internal transition (exit/enter callbacks won't be processed).
-            conditions (string or list): Condition(s) that must pass in order
+            conditions (str or list): Condition(s) that must pass in order
                 for the transition to take place. Either a list providing the
                 name of a callable, or a list of callables. For the transition
                 to occur, ALL callables must return True.
-            unless (string, list): Condition(s) that must return False in order
+            unless (str or list): Condition(s) that must return False in order
                 for the transition to occur. Behaves just like conditions arg
                 otherwise.
-            before (string or list): Callables to call before the transition.
-            after (string or list): Callables to call after the transition.
-            prepare (string or list): Callables to call when the trigger is activated
+            before (str or list): Callables to call before the transition.
+            after (str or list): Callables to call after the transition.
+            prepare (str or list): Callables to call when the trigger is activated
             **kwargs: Additional arguments which can be passed to the created transition.
                 This is useful if you plan to extend Machine.Transition and require more parameters.
         """
+        if trigger == self.model_attribute:
+            raise ValueError("Trigger name cannot be same as model attribute name.")
         if trigger not in self.events:
             self.events[trigger] = self._create_event(trigger, self)
             for model in self.models:
                 self._add_trigger_to_model(trigger, model)
 
-        if isinstance(source, string_types):
-            source = list(self.states.keys()) if source == self.wildcard_all else [source]
+        if source == self.wildcard_all:
+            source = list(self.states.keys())
         else:
-            source = [s.name if self._has_state(s) else s for s in listify(source)]
+            source = [s.name if self._has_state(s) or isinstance(s, Enum) else s for s in listify(source)]
 
         for state in source:
             _dest = state if dest == self.wildcard_same else dest
-            if _dest and self._has_state(_dest):
+            if _dest and self._has_state(_dest) or isinstance(_dest, Enum):
                 _dest = _dest.name
             _trans = self._create_transition(state, _dest, conditions, unless, before,
                                              after, prepare, **kwargs)
@@ -868,23 +900,23 @@ class Machine(object):
                 transitions. E.g., ['A', 'B', 'C'] will generate transitions
                 for A --> B, B --> C, and C --> A (if loop is True). If states
                 is None, all states in the current instance will be used.
-            trigger (string): The name of the trigger method that advances to
+            trigger (str): The name of the trigger method that advances to
                 the next state in the sequence.
             loop (boolean): Whether or not to add a transition from the last
                 state to the first state.
             loop_includes_initial (boolean): If no initial state was defined in
                 the machine, setting this to True will cause the _initial state
                 placeholder to be included in the added transitions.
-            conditions (string or list): Condition(s) that must pass in order
+            conditions (str or list): Condition(s) that must pass in order
                 for the transition to take place. Either a list providing the
                 name of a callable, or a list of callables. For the transition
                 to occur, ALL callables must return True.
-            unless (string, list): Condition(s) that must return False in order
+            unless (str or list): Condition(s) that must return False in order
                 for the transition to occur. Behaves just like conditions arg
                 otherwise.
-            before (string or list): Callables to call before the transition.
-            after (string or list): Callables to call after the transition.
-            prepare (string or list): Callables to call when the trigger is activated
+            before (str or list): Callables to call before the transition.
+            after (str or list): Callables to call after the transition.
+            prepare (str or list): Callables to call when the trigger is activated
             **kwargs: Additional arguments which can be passed to the created transition.
                 This is useful if you plan to extend Machine.Transition and require more parameters.
         """
@@ -928,9 +960,9 @@ class Machine(object):
     def get_transitions(self, trigger="", source="*", dest="*"):
         """ Return the transitions from the Machine.
         Args:
-            trigger (string): Trigger name of the transition.
-            source (string): Limits removal to transitions from a certain state.
-            dest (string): Limits removal to transitions to a certain state.
+            trigger (str): Trigger name of the transition.
+            source (str): Limits removal to transitions from a certain state.
+            dest (str): Limits removal to transitions to a certain state.
         """
         if trigger:
             events = (self.events[trigger], )
@@ -949,9 +981,9 @@ class Machine(object):
     def remove_transition(self, trigger, source="*", dest="*"):
         """ Removes a transition from the Machine and all models.
         Args:
-            trigger (string): Trigger name of the transition.
-            source (string): Limits removal to transitions from a certain state.
-            dest (string): Limits removal to transitions to a certain state.
+            trigger (str): Trigger name of the transition.
+            source (str): Limits removal to transitions from a certain state.
+            dest (str): Limits removal to transitions to a certain state.
         """
         source = listify(source) if source != "*" else source
         dest = listify(dest) if dest != "*" else dest
@@ -989,7 +1021,7 @@ class Machine(object):
             the callable will be resolved from the passed model in event_data. This function is not intended to
             be called directly but through state and transition callback definitions.
         Args:
-            func (string, callable): The callback function.
+            func (str or callable): The callback function.
                 1. First, if the func is callable, just call it
                 2. Second, we try to import string assuming it is a path to a func
                 3. Fallback to a model attribute
@@ -1009,7 +1041,7 @@ class Machine(object):
         """ Converts a model's method name or a path to a callable into a callable.
             If func is not a string it will be returned unaltered.
         Args:
-            func (string, callable): Method name or a path to a callable
+            func (str or callable): Method name or a path to a callable
             event_data (EventData): Currently processed event
         Returns:
             callable function resolved from string or func
