@@ -11,7 +11,8 @@ from ..core import Machine
 
 from .nesting import HierarchicalMachine, NestedTransition, NestedEvent
 from .locking import LockedMachine, LockedEvent
-from .diagrams import GraphMachine, TransitionGraphSupport, NestedGraph
+from .diagrams import GraphMachine, TransitionGraphSupport
+from .markup import MarkupMachine
 
 
 class MachineFactory(object):
@@ -49,13 +50,16 @@ class LockedNestedEvent(LockedEvent, NestedEvent):
     pass
 
 
-class HierarchicalGraphMachine(GraphMachine, HierarchicalMachine):
+class HierarchicalMarkupMachine(MarkupMachine, HierarchicalMachine):
+    pass
+
+
+class HierarchicalGraphMachine(GraphMachine, HierarchicalMarkupMachine):
     """
         A hierarchical state machine with graph support.
     """
 
     transition_cls = NestedGraphTransition
-    graph_cls = NestedGraph
 
 
 class LockedHierarchicalMachine(LockedMachine, HierarchicalMachine):
@@ -73,14 +77,13 @@ class LockedGraphMachine(GraphMachine, LockedMachine):
     pass
 
 
-class LockedHierarchicalGraphMachine(GraphMachine, LockedMachine, HierarchicalMachine):
+class LockedHierarchicalGraphMachine(GraphMachine, LockedMachine, HierarchicalMarkupMachine):
     """
-        A threadsafe hiearchical machine with graph support.
+        A threadsafe hierarchical machine with graph support.
     """
 
     transition_cls = NestedGraphTransition
     event_cls = LockedNestedEvent
-    graph_cls = NestedGraph
 
 
 # 3d tuple (graph, nested, locked)
