@@ -9,14 +9,9 @@
 from copy import copy, deepcopy
 from functools import partial
 import logging
-import warnings
 from six import string_types
 
-from ..core import Machine, Transition, State, Event, listify, MachineError, EventData
-
-# make deprecation warnings of transition visible for module users
-warnings.filterwarnings(action='default', message=r".*transitions version.*")
-
+from ..core import Machine, Transition, State, Event, listify, MachineError, EventData, Enum
 
 _LOGGER = logging.getLogger(__name__)
 _LOGGER.addHandler(logging.NullHandler())
@@ -78,7 +73,9 @@ class NestedState(State):
     """
 
     def __init__(self, name, on_enter=None, on_exit=None, ignore_invalid_triggers=None, parent=None, initial=None):
-        self._name = name
+        if parent is not None and isinstance(name, Enum):
+            raise AttributeError("NestedState does not support nested enumerations.")
+
         self._initial = initial
         self._parent = None
         self.parent = parent
@@ -101,7 +98,10 @@ class NestedState(State):
     def initial(self):
         """ When this state is entered it will automatically enter
             the child with this name if not None. """
-        return self.name + NestedState.separator + self._initial if self._initial else self._initial
+        try:
+            return self.name + NestedState.separator + self._initial if self._initial else self._initial
+        except TypeError:  # we assume an Enum here
+            return self.name + NestedState.separator + self._initial.name
 
     @initial.setter
     def initial(self, value):
@@ -116,12 +116,18 @@ class NestedState(State):
     @property
     def name(self):
         """ The computed name of this state. """
-        return (self.parent.name + NestedState.separator + self._name) if self.parent else self._name
+        if self.parent:
+            return self.parent.name + NestedState.separator + _super(NestedState, self).name
+        return _super(NestedState, self).name
 
     @name.setter
     def name(self, value):
         self._name = value
 
+    @property
+    def value(self):
+        return self.name if isinstance(self._name, string_types) else _super(NestedState, self).value
+
     def is_substate_of(self, state_name):
         """Check whether this state is a substate of a state named `state_name`
         Args:
@@ -131,9 +137,9 @@ class NestedState(State):
         """
 
         temp_state = self
-        while not temp_state.name == state_name and temp_state.level > 0:
+        while not temp_state.value == state_name and temp_state.level > 0:
             temp_state = temp_state.parent
-        return temp_state.name == state_name
+        return temp_state.value == state_name
 
     def exit_nested(self, event_data, target_state):
         """ Tracks child states to exit when the states is exited itself. This should not
@@ -214,10 +220,10 @@ class NestedTransition(Transition):
         machine = event_data.machine
         model = event_data.model
         dest_state = machine.get_state(self.dest)
-        source_state = machine.get_state(model.state)
+        source_state = machine.get_model_state(model)
         lvl = source_state.exit_nested(event_data, dest_state)
         event_data.machine.set_state(self.dest, model)
-        event_data.update(model)
+        event_data.update(dest_state)
         dest_state.enter_nested(event_data, lvl)
 
     # Prevent deep copying of callback lists since these include either references to callable or
@@ -239,13 +245,13 @@ class NestedEvent(Event):
     """ An event type to work with nested states. """
 
     def _trigger(self, model, *args, **kwargs):
-        state = self.machine.get_state(model.state)
+        state = self.machine.get_model_state(model)
         while state.parent and state.name not in self.transitions:
             state = state.parent
         if state.name not in self.transitions:
             msg = "%sCan't trigger event %s from state %s!" % (self.machine.name, self.name,
-                                                               model.state)
-            if self.machine.get_state(model.state).ignore_invalid_triggers:
+                                                               self.machine.get_model_state(model))
+            if self.machine.get_model_state(model).ignore_invalid_triggers:
                 _LOGGER.warning(msg)
             else:
                 raise MachineError(msg)
@@ -276,9 +282,10 @@ class HierarchicalMachine(Machine):
                 assert self._has_state(value)
             state = value
         else:
-            if value not in self.states:
-                self.add_state(value)
-            state = self.get_state(value)
+            state_name = value.name if isinstance(value, Enum) else value
+            if state_name not in self.states:
+                self.add_state(state_name)
+            state = self.get_state(state_name)
         if state.initial:
             self.initial = state.initial
         else:
@@ -312,21 +319,21 @@ class HierarchicalMachine(Machine):
 
         """
         if not allow_substates:
-            return model.state == state_name
+            return getattr(model, self.model_attribute) == state_name
 
-        return self.get_state(model.state).is_substate_of(state_name)
+        return self.get_model_state(model).is_substate_of(state_name)
 
     def _traverse(self, states, on_enter=None, on_exit=None,
                   ignore_invalid_triggers=None, parent=None, remap=None):
         """ Parses passed value to build a nested state structure recursively.
         Args:
-            states (list, string, dict, or State): a list, a State instance, the
+            states (list, str, dict, or State): a list, a State instance, the
                 name of a new state, or a dict with keywords to pass on to the
                 State initializer. If a list, each element can be of any of the
                 latter three types.
-            on_enter (string or list): callbacks to trigger when the state is
+            on_enter (str or list): callbacks to trigger when the state is
                 entered. Only valid if first argument is string.
-            on_exit (string or list): callbacks to trigger when the state is
+            on_exit (str or list): callbacks to trigger when the state is
                 exited. Only valid if first argument is string.
             ignore_invalid_triggers: when True, any calls to trigger methods
                 that are not valid for the present state (e.g., calling an
@@ -336,7 +343,7 @@ class HierarchicalMachine(Machine):
                 argument defined at the Machine level, and is in turn
                 overridden by any ignore_invalid_triggers explicitly
                 passed in an individual state's initialization arguments.
-            parent (NestedState or string): parent state for nested states.
+            parent (NestedState or str): parent state for nested states.
             remap (dict): reassigns transitions named `key from nested machines to parent state `value`.
         Returns: list of new `NestedState` objects
         """
@@ -344,14 +351,14 @@ class HierarchicalMachine(Machine):
         new_states = []
         ignore = ignore_invalid_triggers
         remap = {} if remap is None else remap
-        parent = self.get_state(parent) if isinstance(parent, string_types) else parent
+        parent = self.get_state(parent) if isinstance(parent, (string_types, Enum)) else parent
 
         if ignore is None:
             ignore = self.ignore_invalid_triggers
         for state in states:
             tmp_states = []
             # other state representations are handled almost like in the base class but a parent parameter is added
-            if isinstance(state, string_types):
+            if isinstance(state, (string_types, Enum)):
                 if state in remap:
                     continue
                 tmp_states.append(self._create_state(state, on_enter=on_enter, on_exit=on_exit, parent=parent,
@@ -456,7 +463,7 @@ class HierarchicalMachine(Machine):
                                                      **kwargs)
 
         while self._buffered_transitions:
-            args = self._buffered_transitions.pop()
+            args = self._buffered_transitions.pop(0)
             self.add_transition(**args)
 
     def get_triggers(self, *args):
@@ -508,21 +515,6 @@ class HierarchicalMachine(Machine):
             state_name (str): Name of the destination state.
         """
 
-        # TODO: Remove in 0.7.0
-        if not isinstance(state_name, string_types):
-            warnings.warn("'HierarchicalMachine.to' has been renamed to 'HierarchicalMachine.to_state' and "
-                          "will be removed in transitions version 0.7.0.", DeprecationWarning)
-            model = state_name
-            state_name = args[0]
-            args = tuple(x for idx, x in enumerate(args) if idx > 0)
-
-        event = EventData(self.get_state(model.state), Event('to', self), self,
+        event = EventData(self.get_model_state(model), Event('to', self), self,
                           model, args=args, kwargs=kwargs)
-        self._create_transition(model.state, state_name).execute(event)
-
-    # TODO: Remove in 0.7.0
-    def to(self, model, state_name, *args, **kwargs):
-        """ DEPRECATED! See `to_state` instead """
-        warnings.warn("'HierarchicalMachine.to' has been renamed to 'HierarchicalMachine.to_state' and "
-                      "will be removed in transitions version 0.7.0.", DeprecationWarning)
-        self.to_state(model, state_name, *args, **kwargs)
+        self._create_transition(getattr(model, self.model_attribute), state_name).execute(event)
