@@ -129,7 +129,7 @@ class Volatile(State):
     """ Adds scopes/temporal variables to the otherwise persistent state objects.
     Attributes:
         volatile_cls (cls): Class of the temporal object to be initiated.
-        volatile_hook (string): Model attribute name which will contain the volatile instance.
+        volatile_hook (str): Model attribute name which will contain the volatile instance.
     """
 
     def __init__(self, *args, **kwargs):
@@ -169,7 +169,7 @@ def add_state_features(*args):
             pass
 
         method_list = sum([c.dynamic_methods for c in inspect.getmro(CustomState) if hasattr(c, 'dynamic_methods')], [])
-        CustomState.dynamic_methods = set(method_list)
+        CustomState.dynamic_methods = list(set(method_list))
         cls.state_cls = CustomState
         return cls
     return _class_decorator
